"use strict";
/**
 * patcher for api-extractor, to support legacy export = syntax
 * @reason https://github.com/microsoft/rushstack/issues/2220
 * @solution hack tsHost.readFile for the CompilerState of api-extractor
 *           to replace legacy export = syntax to esm syntax
 */
Object.defineProperty(exports, "__esModule", { value: true });
const api_extractor_1 = require("@microsoft/api-extractor");
const DtsRollupGenerator_1 = require("@microsoft/api-extractor/lib/generators/DtsRollupGenerator");
const IndentedWriter_1 = require("@microsoft/api-extractor/lib/generators/IndentedWriter");
const utils_1 = require("@umijs/utils");
const shared_1 = require("./shared");
// @ts-ignore
const oCreateCompilerHost = api_extractor_1.CompilerState._createCompilerHost;
if (!oCreateCompilerHost.name.includes('father')) {
    // @ts-ignore
    api_extractor_1.CompilerState._createCompilerHost = function _fatherHackCreateCompilerHost(...args) {
        const tsHost = oCreateCompilerHost.apply(api_extractor_1.CompilerState, args);
        const oReadFile = tsHost.readFile;
        // hack readFile method to replace legacy export = syntax to esm
        tsHost.readFile = function fatherHackReadFile(...args) {
            var _a;
            let content = oReadFile.apply(tsHost, args);
            // regexp to match export = [Symbol];
            const legacyExportReg = /[\r\n]export\s+=\s+([\w$]+)\s*([;\r\n])/;
            const exportSymbol = (_a = content === null || content === void 0 ? void 0 : content.match(legacyExportReg)) === null || _a === void 0 ? void 0 : _a[1];
            if (exportSymbol) {
                // replace export = [Symbol] => export { [Symbol] as default } if matched
                content = content.replace(legacyExportReg, '\nexport { $1 as default }$2');
                // double-check export statement
                if (/[\r\n]export\s+=/.test(content)) {
                    utils_1.logger.warn(`Unhandled legacy export syntax in ${utils_1.chalk.gray(args[0])}, please report this issue to father if the d.ts file is unexpected.`);
                }
            }
            return content;
        };
        return tsHost;
    };
    // disable typescript version checking logic to omit the log
    // because api-extractor builtin typescript is not latest
    // @ts-ignore
    api_extractor_1.Extractor._checkCompilerCompatibility = function fatherHackEmpty() { };
    // hijack write file logic
    DtsRollupGenerator_1.DtsRollupGenerator.writeTypingsFile = function fatherHackWriteTypingsFile(collector, dtsFilename, dtsKind) {
        const writer = new IndentedWriter_1.IndentedWriter();
        writer.trimLeadingSpaces = true;
        // @ts-ignore
        DtsRollupGenerator_1.DtsRollupGenerator._generateTypingsFileContent(collector, writer, dtsKind);
        (0, shared_1.setSharedData)(dtsFilename, writer.toString());
    };
}
