"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNestedDepsForPkg = exports.getDepPkgPath = exports.getDtsInfoForPkgPath = exports.getTypeFromPkgJson = void 0;
const utils_1 = require("@umijs/utils");
const path_1 = __importDefault(require("path"));
/**
 * get valid type field value from package.json
 */
function getTypeFromPkgJson(pkg) {
    return pkg.types || pkg.typing || pkg.typings;
}
exports.getTypeFromPkgJson = getTypeFromPkgJson;
/**
 * get d.ts file path and package path for NPM package.json path
 */
function getDtsInfoForPkgPath(pkgPath) {
    const pkg = require(pkgPath);
    const info = { pkgPath: pkgPath, dtsPath: getTypeFromPkgJson(pkg) };
    if (info.dtsPath) {
        // resolve builtin types
        info.dtsPath = path_1.default.resolve(path_1.default.dirname(pkgPath), info.dtsPath);
    }
    else {
        // resolve @types/xxx pkg
        try {
            info.pkgPath = require.resolve(`@types/${pkg.name.replace('@', '').replace('/', '__')}/package.json`, { paths: [pkgPath] });
            info.dtsPath = path_1.default.resolve(path_1.default.dirname(info.pkgPath), getTypeFromPkgJson(require(info.pkgPath)));
        }
        catch {
            return null;
        }
    }
    return info;
}
exports.getDtsInfoForPkgPath = getDtsInfoForPkgPath;
/**
 * get package.json path for specific NPM package
 * @see https://github.com/nodejs/node/issues/33460
 */
function getDepPkgPath(dep, cwd) {
    return utils_1.pkgUp.pkgUpSync({
        cwd: require.resolve(`${dep}/package.json`, { paths: [cwd] }),
    });
}
exports.getDepPkgPath = getDepPkgPath;
/**
 * get all nested dependencies for specific NPM package
 */
function getNestedDepsForPkg(name, cwd, externals, deps = {}) {
    if (name in deps || externals[name])
        return deps;
    const pkgPath = getDepPkgPath(name, cwd);
    const pkgJson = require(pkgPath);
    const pkgDeps = pkgJson.dependencies || {};
    deps[name] = pkgJson.version;
    Object.keys(pkgDeps).forEach((name) => {
        getNestedDepsForPkg(name, pkgPath, externals, deps);
    });
    return deps;
}
exports.getNestedDepsForPkg = getNestedDepsForPkg;
